import argparse
import mutagen.id3
import mutagen.mp3
import rainwave_tools.utils


def log(m):
    print(m)


TAG_SPEC = {'album': 'TALB', 'apic': 'APIC', 'art': 'APIC', 'artist': 'TPE1', 'artist2': 'TPE2', 'bpm': 'TBPM',
            'comm': 'COMM', 'comment': 'COMM', 'composer': 'TCOM', 'disc': 'TPOS', 'encoder': 'TSSE', 'genre': 'TCON',
            'isrc': 'TSRC', 'lyric': 'USLT', 'popm': 'POPM', 'priv': 'PRIV', 'private': 'PRIV', 'rva2': 'RVA2',
            'talb': 'TALB', 'tbpm': 'TBPM', 'tcmp': 'TCMP', 'tcom': 'TCOM', 'tcon': 'TCON', 'tcop': 'TCOP',
            'tdrc': 'TDRC', 'tdrl': 'TDRL', 'tdtg': 'TDTG', 'tenc': 'TENC', 'text': 'TXXX', 'tflt': 'TFLT',
            'tit1': 'TIT1', 'tit2': 'TIT2', 'tit3': 'TIT3', 'title': 'TIT2', 'tmed': 'TMED', 'toal': 'TOAL',
            'tope': 'TOPE', 'tpe1': 'TPE1', 'tpe2': 'TPE2', 'tpos': 'TPOS', 'tpub': 'TPUB', 'track': 'TRCK',
            'trck': 'TRCK', 'tsrc': 'TSRC', 'tsse': 'TSSE', 'tsst': 'TSST', 'txxx': 'TXXX', 'ufid': 'UFID',
            'uslt': 'USLT', 'wcom': 'WCOM', 'woaf': 'WOAF', 'woar': 'WOAR', 'www': 'WXXX', 'wxxx': 'WXXX',
            'year': 'TDRC'}


def tag_drop(args):
    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        try:
            _md = mutagen.id3.ID3(str(mp3))
        except mutagen.id3.ID3NoHeaderError:
            _md = mutagen.id3.ID3()
        _tag = TAG_SPEC.get(args.tag, args.tag)
        _md.delall(_tag)
        try:
            _md.save()
        except IOError as _ioe:
            log(f'ERROR : {_ioe}')
            continue
        log(f'{mp3} : dropped all tags of type {args.tag!r}')


def tag_dump(args):
    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        _md = mutagen.id3.ID3(str(mp3))
        log(_md.pprint())
        log('---------')


def tag_set(args):
    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        try:
            _md = mutagen.id3.ID3(str(mp3))
        except mutagen.id3.ID3NoHeaderError:
            _md = mutagen.id3.ID3()
        _tag = TAG_SPEC.get(args.tag, args.tag)
        if _tag in ['COMM', 'TALB', 'TCON', 'TDRC', 'TIT2', 'TPE1', 'TPOS', 'TRCK']:
            _md.delall(_tag)
            tag_class = getattr(mutagen.id3, _tag)
            _md.add(tag_class(encoding=3, text=[args.value]))
        elif _tag == 'WXXX':
            _md.delall(_tag)
            _md.add(mutagen.id3.WXXX(encoding=0, url=args.value))
        _md.save(str(mp3))
        log(f'{mp3}: {args.tag} set to {args.value!r}')


def tag_show(args):
    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        _audio = mutagen.mp3.MP3(str(mp3))
        log(f'file    : {mp3}')
        log(f'length  : {int(_audio.info.length)} seconds')
        _md = mutagen.id3.ID3(str(mp3))

        for _frame in _md.getall('TALB'):
            for _text in _frame.text:
                log(f'album   : {_text}')

        for _frame in _md.getall('TIT2'):
            for _text in _frame:
                log(f'title   : {_text}')

        for _frame in _md.getall('TPE1'):
            for _text in _frame.text:
                log(f'artist  : {_text}')

        for _frame in _md.getall('TCON'):
            for _text in _frame:
                log(f'genre   : {_text}')

        for _frame in _md.getall('TRCK'):
            for _text in _frame:
                log(f'track   : {_text}')

        for _frame in _md.getall('TPOS'):
            for _text in _frame:
                log(f'disc    : {_text}')

        for _frame in _md.getall('WXXX'):
            log(f'www     : {_frame.url}')

        for _frame in _md.getall('COMM'):
            for _text in _frame:
                log(f'comment : {_text}')

        for _frame in _md.getall('TDRC'):
            for _text in _frame:
                log(f'year    : {_text}')
        log('---------')


def parse_args():
    ap = argparse.ArgumentParser(description='View and edit ID3 tags on MP3 files.')

    sp_desc = 'Specify one of the following commands. To get help on an individual command, use \'rwtag <command> -h\'.'
    sp = ap.add_subparsers(title='Available commands', description=sp_desc, dest='command')
    sp.required = True

    ps_drop = sp.add_parser('drop', description='Remove a tag from one or more MP3 files.', aliases=['remove', 'rm'])
    ps_drop.add_argument('tag', help='The name of the tag to remove.')
    ps_drop.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_drop.set_defaults(func=tag_drop)

    ps_dump = sp.add_parser('dump', description='Show all tags on one or more MP3 files.')
    ps_dump.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_dump.set_defaults(func=tag_dump)

    ps_set = sp.add_parser('set', description='Set a tag to a certain value on one or more MP3 files.')
    ps_set.add_argument('tag', help='The name of the tag to set.')
    ps_set.add_argument('value', help='The value to set for the tag.')
    ps_set.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_set.set_defaults(func=tag_set)

    ps_show = sp.add_parser('show', description='Show only tags that Rainwave cares about on one or more MP3 files.')
    ps_show.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_show.set_defaults(func=tag_show)

    return ap.parse_args()


def main():
    args = parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



import mutagen.id3
import os

GENRE_FILE = '_genre.txt'


def log(m):
    print(m)


def main():
    cwd = os.getcwd()
    log(f'{cwd} : current directory')

    for root, folders, files in os.walk(cwd):
        used_genre = ''
        has_mp3s = False
        for filename in files:
            if filename.endswith('.mp3'):
                has_mp3s = True
                if not used_genre:
                    mp3_path = os.path.join(root, filename)
                    tags = mutagen.id3.ID3(mp3_path)
                    genre_frames = tags.getall('TCON')
                    for frame in genre_frames:
                        used_genre = frame.text[0]
                        break
        if not has_mp3s:
            log(f'{root} : no mp3s')
            continue

        this_genre_file = os.path.join(root, GENRE_FILE)
        if GENRE_FILE in files:
            with open(this_genre_file) as f:
                genre = f.readline().strip()
                log(f'{this_genre_file} : {genre}')
            continue
        try:
            new_genre = input(f'{root} : set genre [{used_genre}] > ')
            if new_genre == '':
                continue
            with open(this_genre_file, 'w') as f:
                f.write(f'{new_genre}\n')
        except KeyboardInterrupt:
            print('')
            break


if __name__ == '__main__':
    main()



import argparse
import logging
import mutagen.id3
import rainwave_tools
import sys

log = logging.getLogger(__name__)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    logging.basicConfig(stream=sys.stdout, level='INFO', format='%(message)s')
    args = parse_args()

    change_count = 0

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        tags = mutagen.id3.ID3(str(mp3))
        titles = tags.getall('TIT2')
        if titles:
            title = tags.getall('TIT2')[0].text[0]
        else:
            title = ''
        new_title = input(f'{mp3} [{title}] > ')
        if new_title:
            change_count += 1
            tags.delall('TIT2')
            tags.add(mutagen.id3.TIT2(encoding=3, text=[new_title]))
            tags.save()

    m = f'** updated tags in {change_count} file'
    if change_count != 1:
        m = f'{m}s'
    log.info(m)


if __name__ == '__main__':
    main()



import os
import pathlib
import psycopg2
import sys


def main():
    if 'RW_DB_PASS' in os.environ:
        rw_db_pass = os.environ.get('RW_DB_PASS')
    else:
        print('Please set the RW_DB_PASS environment variable.')
        sys.exit()

    cnx_string = f'dbname=rainwave user=orpheus password={rw_db_pass}'
    cnx = psycopg2.connect(cnx_string)

    sql = '''
        SELECT DISTINCT
            album_id,
            album_name
        FROM r4_songs
        JOIN r4_albums USING (album_id)
        WHERE song_verified IS TRUE
    '''
    with cnx.cursor() as cur:
        cur.execute(sql)
        rows = cur.fetchall()

    art_dir = pathlib.Path('/var/www/album_art')

    for row in rows:
        album_id = str(row[0])
        album_name = str(row[1])
        album_fn = f'a_{album_id}_120.jpg'
        if not art_dir.joinpath(album_fn).exists():
            print(album_name)


if __name__ == '__main__':
    main()



import argparse
import os
import psycopg2
import sys


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('query')
    return parser.parse_args()


def main():
    args = parse_args()

    if 'RW_DB_PASS' in os.environ:
        rw_db_pass = os.environ.get('RW_DB_PASS')
    else:
        print('Please set the RW_DB_PASS environment variable.')
        sys.exit()

    conn = psycopg2.connect(f'dbname=rainwave user=orpheus password={rw_db_pass}')

    with conn.cursor() as cur:
        sql = '''
            select
                a.album_id,
                a.album_name,
                array_agg(distinct left(s.song_filename, 0 - position('/' in reverse(s.song_filename)))) album_folders
            from r4_songs s
            join r4_albums a on a.album_id  = s.album_id
            where s.song_verified is true
            and a.album_name ilike %s
            group by a.album_id, a.album_name
            order by a.album_name
        '''
        cur.execute(sql, [f'%{args.query}%'])
        rows = cur.fetchall()

    for row in rows:
        album_id = int(row[0])
        album_name = row[1]
        album_folders = sorted(row[2])
        print(f'{album_id} // {album_name}')
        for f in album_folders:
            print(f'  {f}')


if __name__ == '__main__':
    main()



import argparse
import os
import psycopg2
import sys


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('old_song_id')
    parser.add_argument('new_song_id')
    parser.add_argument('--live', action='store_true')
    return parser.parse_args()


def main():
    args = parse_args()

    rw_db_pass = os.getenv('RW_DB_PASS')
    if rw_db_pass is None:
        print('Please set the RW_DB_PASS environment variable.')
        sys.exit()

    cnx = psycopg2.connect(f'postgres://orpheus:{rw_db_pass}@localhost/rainwave')
    with cnx.cursor() as cur:
        # Look up album and title for old and new songs
        sql = '''
            SELECT album_name, song_title, song_filename
            FROM r4_songs
            JOIN r4_albums USING (album_id)
            WHERE song_id = %(song_id)s
        '''
        cur.execute(sql, {'song_id': args.old_song_id})
        old_song = cur.fetchone()
        print(f'Moving ratings from : {old_song[0]} / {old_song[1]}')
        cur.execute(sql, {'song_id': args.new_song_id})
        new_song = cur.fetchone()
        print(f'Moving ratings to   : {new_song[0]} / {new_song[1]}')

        # Get all ratings for old song
        sql = '''
            SELECT user_id, song_rating_user, song_rated_at
            FROM r4_song_ratings
            WHERE song_id = %(song_id)s
              AND song_rating_user IS NOT NULL
        '''
        cur.execute(sql, {'song_id': args.old_song_id})
        rows = cur.fetchall()
        print(f'Found {len(rows)} ratings for old song')

        moved_ratings = 0
        new_song_already_rated = 0
        for old_song_rating in rows:
            # See if this user has a rating for the new song
            sql = '''
                SELECT song_rating_user, song_rated_at
                FROM r4_song_ratings
                WHERE song_id = %(song_id)s
                  AND user_id = %(user_id)s
            '''
            cur.execute(sql, {'song_id': args.new_song_id, 'user_id': old_song_rating[0]})
            new_song_rating = cur.fetchone()

            if new_song_rating is None:
                # New song was never rated or faved
                sql = '''
                    UPDATE r4_song_ratings
                    SET song_id = %(new_song_id)s
                    WHERE song_id = %(old_song_id)s AND user_id = %(user_id)s
                '''
                params = {
                    'new_song_id': args.new_song_id,
                    'old_song_id': args.old_song_id,
                    'user_id': old_song_rating[0]
                }
                cur.execute(sql, params)
                moved_ratings += 1
            elif new_song_rating[0] is None:
                # New song was faved but not rated
                sql = '''
                    UPDATE r4_song_ratings
                    SET song_rating_user = %(song_rating_user)s, song_rated_at = %(song_rated_at)s
                    WHERE song_id = %(song_id)s AND user_id = %(user_id)s
                '''
                params = {
                    'song_rating_user': old_song_rating[1],
                    'song_rated_at': old_song_rating[2],
                    'song_id': args.new_song_id,
                    'user_id': old_song_rating[0]
                }
                cur.execute(sql, params)
                moved_ratings += 1
            else:
                # New song already has a rating
                new_song_already_rated += 1

    if args.live:
        cnx.commit()
    else:
        print('**')
        print('** No changes will be made in the database. Use --live to commit changes.')
        print('**')

    print(f'Users where rating for old song was applied to new song: {moved_ratings}')
    print(f'Users that already had a rating for the new song: {new_song_already_rated}')


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import pathlib
import rainwave_tools.ocremix
import shutil
import stat
import urllib.request

COMMENT = 'Get @ OCR'
GENRE_PROMPT = 'Enter a genre > '


def log(m):
    print(m)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--destination', default='/icecast/staging', type=pathlib.Path)
    parser.add_argument('ocr_num', type=int)
    return parser.parse_args()


def main():
    args = parse_args()

    log(f'Processing OCR{args.ocr_num:05}')
    remix = rainwave_tools.ocremix.OCReMix(args.ocr_num)

    log(f'Downloading mp3 file from {remix.mp3_url}')
    temp_file, _ = urllib.request.urlretrieve(remix.mp3_url)
    log(f'Downloaded mp3 file to {temp_file}')

    tags = mutagen.id3.ID3(temp_file)

    log(f'Setting TALB (album) to {remix.album!r}')
    tags.delall('TALB')
    tags.add(mutagen.id3.TALB(encoding=3, text=[remix.album]))

    log(f'Setting TIT2 (title) to {remix.title!r}')
    tags.delall('TIT2')
    tags.add(mutagen.id3.TIT2(encoding=3, text=[remix.title]))

    log(f'Setting TPE1 (artist) to {remix.artist!r}')
    tags.delall('TPE1')
    tags.add(mutagen.id3.TPE1(encoding=3, text=[remix.artist]))

    log(f'Setting WXXX (www) to {remix.info_url!r}')
    tags.delall('WXXX')
    tags.add(mutagen.id3.WXXX(encoding=0, url=remix.info_url))

    log(f'Setting COMM (comment) to {COMMENT!r}')
    tags.delall('COMM')
    tags.add(mutagen.id3.COMM(encoding=3, text=[COMMENT]))

    if remix.has_lyrics:
        log('** This remix has lyrics!')

    tags.delall('TCON')
    genre = input(GENRE_PROMPT)
    if genre:
        log(f'Setting TCON (genre) to {genre!r}')
        tags.add(mutagen.id3.TCON(encoding=3, text=[genre]))

    log('Dropping unnecessary tags')
    for tag in ['APIC', 'TCMP', 'TCOM', 'TCOP', 'TDRC', 'TENC', 'TIT1', 'TIT3', 'TOAL', 'TOPE', 'TPE2', 'TPUB', 'TRCK',
                'TSSE', 'TXXX', 'USLT', 'WOAR']:
        tags.delall(tag)

    tags.save(temp_file)

    dest_folder = args.destination / remix.safe_album
    dest_folder.mkdir(parents=True, exist_ok=True)
    dest_file = dest_folder / f'{remix.safe_title}.mp3'
    log(f'Moving {temp_file} to {dest_file}')
    final = shutil.move(temp_file, dest_file)

    log('Changing file permissions')
    perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH
    final.chmod(perms)

    log('Cleaning up temporary files')
    urllib.request.urlcleanup()


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.utils


def log(m):
    print(f'{m}')


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', '--include-album', action='store_true')
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    args = parse_args()

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        log(f'** {mp3}')
        tags = mutagen.id3.ID3(str(mp3))
        title = tags.getall('TIT2')[0].text[0]
        safe_title = rainwave_tools.utils.make_safe(title)
        new_name = f'{safe_title}.mp3'
        if args.include_album:
            album = tags.getall('TALB')[0].text[0]
            safe_album = rainwave_tools.utils.make_safe(album)
            new_name = f'{safe_album}_{new_name}'
        new_file = mp3.with_name(new_name)
        log(f'-> {new_file}')
        mp3.rename(new_file)


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.utils


def log(m):
    print(m)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('old_name')
    parser.add_argument('new_name')
    parser.add_argument('path', nargs='*', default='.', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    args = parse_args()
    change_count = 0
    errors = []

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        changed = False
        tags = mutagen.id3.ID3(str(mp3))
        artist_tag = tags.getall('TPE1')[0].text[0]
        artists = [a.strip() for a in artist_tag.split(',')]
        for i, artist in enumerate(artists):
            if artist == args.old_name:
                artists[i] = args.new_name
                changed = True
        if changed:
            artist_tag = ', '.join(artists)
            tags.delall('TPE1')
            tags.add(mutagen.id3.TPE1(encoding=3, text=[artist_tag]))
            try:
                tags.save()
            except PermissionError as e:
                errors.append(e)
            else:
                change_count += 1
                log(f'{mp3} : new artist tag {artist_tag!r}')

    m = f'** updated tags in {change_count} file'
    if change_count != 1:
        m = f'{m}s'
    log(m)
    if errors:
        log('**********')
        log('* ERRORS *')
        log('**********')
        for error in errors:
            log(error)


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.utils


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    args = parse_args()
    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        tags = mutagen.id3.ID3(str(mp3))
        tit3 = tags.getall('TIT3')[0].text[0]
        tags.delall('TIT2')
        tags.add(mutagen.id3.TIT2(encoding=3, text=[tit3]))
        tags.save()


if __name__ == '__main__':
    main()



import lxml.html
import rainwave_tools.utils
import urllib.request


class OCReMix(object):
    INFO_URL_TEMPLATE = 'https://ocremix.org/remix/OCR{:05}'

    def __init__(self, ocr_id):
        self.ocr_id = ocr_id
        self.info_url = self.INFO_URL_TEMPLATE.format(self.ocr_id)
        self._tree = None
        self._album = None
        self._safe_album = None
        self._title = None
        self._safe_title = None
        self._artist = None
        self._mp3_url = None
        self._has_lyrics = None
        self._tags = []

    def load_from_url(self):
        data = urllib.request.urlopen(self.info_url)
        page = data.read().decode()
        self._tree = lxml.html.fromstring(page)

    @property
    def album(self):
        if self._album is None:
            if self._tree is None:
                self.load_from_url()
            self._album = self._tree.xpath('//h1/a')[0].text
        return self._album

    @property
    def safe_album(self):
        if self._safe_album is None:
            self._safe_album = rainwave_tools.utils.make_safe(self.album)
        return self._safe_album

    @property
    def title(self):
        if self._title is None:
            if self._tree is None:
                self.load_from_url()
            self._title = self._tree.xpath('//h1/a')[0].tail[2:-2]
        self._title = self._title.replace('\ufeff', '')
        return self._title

    @property
    def safe_title(self):
        if self._safe_title is None:
            self._safe_title = rainwave_tools.utils.make_safe(self.title)
        return self._safe_title

    @property
    def artist(self):
        if self._artist is None:
            if self._tree is None:
                self.load_from_url()
            self._artist = ', '.join([a.text.replace('\ufeff', '')
                                      for a in self._tree.xpath('//h2/a[starts-with(@href, "/artist")]')])
        return self._artist

    @property
    def mp3_url(self):
        if self._mp3_url is None:
            if self._tree is None:
                self.load_from_url()
            _xpath = '//div[@id="modalDownload"]//a[contains(@href, "ocrmirror.org")]/@href'
            self._mp3_url = self._tree.xpath(_xpath)[0]
        return self._mp3_url

    @property
    def has_lyrics(self):
        if self._has_lyrics is None:
            if self._tree is None:
                self.load_from_url()
            self._has_lyrics = bool(self._tree.xpath('//a[@href="#tab-lyrics"]'))
        return self._has_lyrics

    @property
    def tags(self):
        if not self._tags:
            xpath = '//*[@id="main-content"]/div[1]/div/div[1]/section[1]/div/div/section[2]/div[2]/section[3]/div/span'
            for t in self._tree.xpath(xpath):
                self._tags.append(t.text)
            self._tags.sort()
        return self._tags



import argparse
import mutagen.id3
import rainwave_tools.utils


def log(m):
    print(m)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    args = parse_args()

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        tags = mutagen.id3.ID3(str(mp3))
        tags.delall('RVA2')
        tags.delall('TXXX:MP3GAIN_MINMAX')
        tags.delall('TXXX:replaygain_album_gain')
        tags.delall('TXXX:replaygain_album_peak')
        tags.delall('TXXX:replaygain_reference_loudness')
        tags.delall('TXXX:replaygain_track_gain')
        tags.delall('TXXX:replaygain_track_peak')
        tags.save()
        log('** wiped replaygain tags from {}'.format(mp3))


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.ocremix
import rainwave_tools.utils
import re


def log(m):
    print(m)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--titles', action='store_true', help='update titles')
    parser.add_argument('-u', '--urls', action='store_true', help='update URLs')
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def get_url(tags: mutagen.id3.ID3):
    for frame in tags.getall('WXXX'):
        return frame.url


def main():
    args = parse_args()

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        ocr_id = None
        changed = False
        tags = mutagen.id3.ID3(str(mp3))
        url = get_url(tags)
        if url:
            for match in re.findall('OCR\d{5}', url):
                ocr_id = int(match[3:])
        if ocr_id is None:
            log(f'{mp3} : not an OCR url')
            continue
        remix = rainwave_tools.ocremix.OCReMix(ocr_id)

        if args.urls:
            if remix.info_url != url:
                log(f'{mp3} : updating url to {remix.info_url}')
                tags.delall('WXXX')
                tags.add(mutagen.id3.WXXX(encoding=0, url=remix.info_url))
                changed = True

        if args.titles:
            tag_title = tags.getall('TIT2')[0][0]
            if remix.title != tag_title:
                log(f'{mp3} : updating title to {remix.title}')
                tags.delall('TIT2')
                tags.add(mutagen.id3.TIT2(encoding=3, text=[remix.title]))
                changed = True

        if changed:
            tags.save()


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.utils


def log(message):
    print(f'** {message}')


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    return parser.parse_args()


def main():
    args = parse_args()

    for mp3 in rainwave_tools.utils.get_mp3s(args.path):
        log(mp3)
        tags = mutagen.id3.ID3(str(mp3))

        album = tags.getall('TALB')[0].text[0]
        new_album = input(f'album [{album}] > ')
        if new_album:
            tags.delall('TALB')
            tags.add(mutagen.id3.TALB(encoding=3, text=[new_album]))
            tags.save()

        title = tags.getall('TIT2')[0].text[0]
        new_title = input(f'title [{title}] > ')
        if new_title:
            tags.delall('TIT2')
            tags.add(mutagen.id3.TIT2(encoding=3, text=[new_title]))
            tags.save()

        artist = tags.getall('TPE1')[0].text[0]
        new_artist = input(f'artist [{artist}] > ')
        if new_artist:
            tags.delall('TPE1')
            tags.add(mutagen.id3.TPE1(encoding=3, text=[new_artist]))
            tags.save()


if __name__ == '__main__':
    main()



import pathlib

from typing import Any

__all__ = ['get_mp3s', 'make_safe', 'path_help']


def get_mp3s(paths: Any):
    if not isinstance(paths, list):
        paths = [paths]
    for path in paths:
        if isinstance(path, pathlib.Path):
            p = path.resolve()
        else:
            p = pathlib.Path(path).resolve()
        if p.is_dir():
            for item in p.iterdir():
                for mp3 in get_mp3s(item):
                    yield mp3
        else:
            if p.suffix.lower() == '.mp3':
                yield p


# Special characters sorted by results of ord()
# {
#  '²': 178
#  'É': 201,
#  'Ü': 220,
#  'à': 224,
#  'á': 225,
#  'â': 226,
#  'ã': 227,
#  'ä': 228,
#  'ç': 231,
#  'è': 232,
#  'é': 233,
#  'ê': 234,
#  'í': 237,
#  'ð', 240,
#  'ñ': 241,
#  'ó': 243,
#  'ö': 246,
#  'ú': 250,
#  'ü': 252,
#  'ş': 351,
#  'К': 1050,
#  'С': 1057,
#  'а': 1072,
#  'в': 1074,
#  'е': 1077,
#  'и': 1080,
#  'й': 1081,
#  'к': 1082,
#  'м': 1084,
#  'н': 1085,
#  'о': 1086,
#  'с': 1089,
#  'т': 1090
# }

def make_safe(s):
    translate_table = {ord(char): None for char in ' !"#%&\'()*+,-./:;<=>?@[\\]^_`{|}~–—あいごま고말싶은하'}
    special = dict(zip(map(ord, '²ÉÜàáâãäçèéêíðñóöúüşКСавеийкмност'), '2EUaaaaaceeeidnoouusKSaveijkmnost'))
    translate_table.update(special)
    return s.translate(translate_table)


path_help = ("A file or directory to process. If you specify a directory, all files and subdirectories in the "
             "directory will be processed recursively. Only files with the extension '.mp3' will be processed. You may "
             "specify more than one file or directory.")



import mutagen.id3
import os

GENRE_FILE = '_genre.txt'


def log(m):
    print(m)


def main():
    cwd = os.getcwd()
    log(f'{cwd} : current directory')

    for root, folders, files in os.walk(cwd):
        for filename in files:
            if filename.endswith('.mp3'):
                break
        else:
            log(f'{root} : no mp3s')
            continue

        this_genre_file = os.path.join(root, GENRE_FILE)
        if GENRE_FILE in files:
            with open(this_genre_file) as f:
                genre = f.readline().strip()
                log(f'{this_genre_file} : {genre}')
        else:
            log(f'{this_genre_file} : not found')
            continue

        if genre == '':
            log(f'{this_genre_file} : empty, skipping this folder')
            continue

        for filename in files:
            if filename.endswith('.mp3'):
                mp3_path = os.path.join(root, filename)
                tags = mutagen.id3.ID3(mp3_path)
                cur_genre = ''
                for genre_tag in tags.getall('TCON'):
                    for genre_text in genre_tag.text:
                        cur_genre = genre_text
                    break
                if cur_genre == genre:
                    log(f'{mp3_path} already has genre {cur_genre}')
                    continue
                else:
                    tags.delall('TCON')
                    tags.add(mutagen.id3.TCON(encoding=3, text=[genre]))
                    tags.save()
                    log(f'{mp3_path} : setting genre to {genre}')


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3


def log(m):
    print(m)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('file')
    parser.add_argument('url')
    return parser.parse_args()


def main():
    args = parse_args()

    tags = mutagen.id3.ID3(args.file)
    log(f'{args.file} : setting www to {args.url}')
    tags.delall('WXXX')
    tags.add(mutagen.id3.WXXX(encoding=0, url=args.url))
    comment = 'Get @ OCR'
    log(f'{args.file} : setting comment to {comment!r}')
    tags.delall('COMM')
    tags.add(mutagen.id3.COMM(encoding=3, text=[comment]))
    tags.save()


if __name__ == '__main__':
    main()



import argparse
import mutagen.id3
import rainwave_tools.utils
import sys


def get_groups(path):
    rv = set()
    tags = mutagen.id3.ID3(str(path))
    for group_tag in tags.getall('TCON'):
        for group_text in group_tag.text:
            rv |= set([a.strip() for a in group_text.split(',')])
    return rv


def set_groups(path, groups=None):
    if groups is None:
        groups = set()
    tags = mutagen.id3.ID3(str(path))
    tags.delall('TCON')
    if groups:
        group_tag = ', '.join(groups)
        tags.add(mutagen.id3.TCON(encoding=3, text=[group_tag]))
    tags.save()
    print(f'{path} : cooldown groups: {list(groups)!r}')


def cdg_add(args):
    errors = []
    for f in rainwave_tools.utils.get_mp3s(args.path):
        cdgs = get_groups(f)
        if args.group not in cdgs:
            cdgs.add(args.group)
            try:
                set_groups(f, cdgs)
            except PermissionError as e:
                errors.append(e)
    return errors


def cdg_drop(args):
    errors = []
    for f in rainwave_tools.utils.get_mp3s(args.path):
        cdgs = get_groups(f)
        if args.group in cdgs:
            cdgs.discard(args.group)
            try:
                set_groups(f, cdgs)
            except PermissionError as e:
                errors.append(e)
    return errors


def cdg_find(args):
    errors = []
    for f in rainwave_tools.utils.get_mp3s(args.path):
        try:
            cdgs = get_groups(f)
        except mutagen.id3.ID3NoHeaderError as e:
            errors.append(e)
            continue
        if args.group in cdgs:
            if args.print0:
                print(f, end='\x00')
            else:
                print(f)
    return errors


def cdg_list(args):
    errors = []
    for f in rainwave_tools.utils.get_mp3s(args.path):
        try:
            cdgs = get_groups(f)
        except mutagen.id3.ID3NoHeaderError as e:
            errors.append(e)
            continue
        print(f'{f} : {list(cdgs)}')
    return errors


def cdg_rename(args):
    errors = []
    for f in rainwave_tools.utils.get_mp3s(args.path):
        cdgs = get_groups(f)
        if args.old_group in cdgs:
            cdgs.discard(args.old_group)
            cdgs.add(args.new_group)
            try:
                set_groups(f, cdgs)
            except PermissionError as e:
                errors.append(e)
    return errors


def parse_args():
    ap = argparse.ArgumentParser(description='Manage cooldown groups (genre tags) in mp3 files')
    sp = ap.add_subparsers(dest='command', title='commands')
    sp.required = True

    ls_help = 'Show the current cooldown groups for one or more mp3 files'
    ps_ls = sp.add_parser('ls', aliases=['list'], help=ls_help, description=ls_help)
    ps_ls.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_ls.set_defaults(func=cdg_list)

    find_help = 'Find mp3 files that belong to the specified cooldown group'
    find_print0_help = 'Separate file names with a null character instead of a newline'
    ps_find = sp.add_parser('find', aliases=['search'], help=find_help, description=find_help)
    ps_find.add_argument('-0', '--print0', action='store_true', help=find_print0_help)
    ps_find.add_argument('group', help='The cooldown group to search for')
    ps_find.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_find.set_defaults(func=cdg_find)

    add_help = 'Add a cooldown group to one or more mp3 files'
    ps_add = sp.add_parser('add', help=add_help, description=add_help)
    ps_add.add_argument('group', help='The cooldown group to add to the specified files')
    ps_add.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_add.set_defaults(func=cdg_add)

    rm_help = 'Remove a cooldown group from one or more mp3 files'
    ps_rm = sp.add_parser('rm', aliases=['drop', 'remove'], help=rm_help, description=rm_help)
    ps_rm.add_argument('group', help='The cooldown group to remove from the specified files')
    ps_rm.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_rm.set_defaults(func=cdg_drop)

    mv_help = 'Rename a cooldown group in one or more mp3 files'
    ps_mv = sp.add_parser('mv', aliases=['rename', 'replace'], help=mv_help, description=mv_help)
    ps_mv.add_argument('old_group', help='The current name of the cooldown group that you want to rename')
    ps_mv.add_argument('new_group', help='The new name for the cooldown group that you want to rename')
    ps_mv.add_argument('path', nargs='+', help=rainwave_tools.utils.path_help)
    ps_mv.set_defaults(func=cdg_rename)

    return ap.parse_args()


def main():
    args = parse_args()
    errors = args.func(args)
    if errors:
        print('**********', file=sys.stderr)
        print('* ERRORS *', file=sys.stderr)
        print('**********', file=sys.stderr)
        for error in errors:
            print(error, file=sys.stderr)


if __name__ == '__main__':
    main()



import argparse
import json
import mutagen.id3
import os
import pathlib
import psycopg2
import requests
import sys

from requests.exceptions import ConnectionError, MissingSchema, ReadTimeout
from urllib3.exceptions import ReadTimeoutError


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--skip', help='skip URLs that match this substring')
    return parser.parse_args()


def get_config():
    data = {}
    home = pathlib.Path(os.environ.get('HOME')).resolve()
    config = home / '.config/rainwave_tools/url_check.json'
    if not config.parent.exists():
        config.parent.mkdir(parents=True)
    if config.exists():
        with config.open() as f:
            data = json.load(f)
    return data


def set_config(c):
    home = pathlib.Path(os.environ.get('HOME')).resolve()
    config = home / '.config/rainwave_tools/url_check.json'
    if not config.parent.exists():
        config.parent.mkdir(parents=True)
    with config.open('w') as f:
        json.dump(c, f, indent=1)


def get_files_with_url(cnx, url):
    files = []
    sql = '''
        SELECT DISTINCT song_filename
        FROM r4_songs
        WHERE song_verified IS TRUE AND song_url = %s
        ORDER BY song_filename
    '''
    with cnx.cursor() as cur:
        cur.execute(sql, [url])
        rows = cur.fetchall()

    for row in rows:
        files.append(str(row[0]))

    return files


def replace_url(cnx, old_url, new_url):
    sql = '''
        SELECT DISTINCT song_filename
        FROM r4_songs
        WHERE song_verified IS TRUE AND song_url = %s
        ORDER BY song_filename
    '''
    with cnx.cursor() as cur:
        cur.execute(sql, [old_url])
        files = cur.fetchall()

    for file in files:
        filename = str(file[0])
        tags = mutagen.id3.ID3(filename)
        tags.delall('WXXX')
        tags.add(mutagen.id3.WXXX(encoding=0, url=new_url))
        tags.save()
        print(f'{filename} : new www {new_url!r}')


def main():
    args = parse_args()

    if args.skip:
        print(f'Skipping URLS that match: {args.skip}')

    if 'RW_DB_PASS' in os.environ:
        rw_db_pass = os.environ.get('RW_DB_PASS')
    else:
        print('Please set the RW_DB_PASS environment variable.')
        sys.exit()

    c = get_config()

    cnx = psycopg2.connect(f'dbname=rainwave user=orpheus password={rw_db_pass}')

    sql = 'SELECT DISTINCT song_url FROM r4_songs WHERE song_verified IS TRUE ORDER BY song_url'
    with cnx.cursor() as cur:
        cur.execute(sql)
        urls = cur.fetchall()

    count = 0
    for row in urls:
        count += 1
        print(f'{count}\r', end='')
        url = str(row[0])
        if url is None or url in c.get('good_urls', []):
            continue
        if args.skip and args.skip in url:
            continue
        try:
            resp = requests.head(url, timeout=1)
            if resp.status_code in [403, 405]:
                resp = requests.get(url, timeout=1)
            if resp.status_code == 200:
                good_urls = c.get('good_urls', [])
                good_urls.append(url)
                c['good_urls'] = good_urls
                set_config(c)
                continue
            else:
                code = resp.status_code
        except (MissingSchema, ConnectionError, ReadTimeout, ReadTimeoutError, TimeoutError):
            code = '---'
        while True:
            new_url = input('{} {} > '.format(code, url))
            if new_url == '?':
                for file in get_files_with_url(cnx, url):
                    print('  * ' + file)
            elif new_url:
                replace_url(cnx, url, new_url)
                break
            else:
                break


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



import argparse
import os
import psycopg2
import sys


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('query')
    return parser.parse_args()


def main():
    args = parse_args()

    if 'RW_DB_PASS' in os.environ:
        rw_db_pass = os.environ.get('RW_DB_PASS')
    else:
        print('Please set the RW_DB_PASS environment variable.')
        sys.exit()

    conn = psycopg2.connect(f'dbname=rainwave user=orpheus password={rw_db_pass}')

    with conn.cursor() as cur:
        sql = '''
            SELECT
                song_id,
                album_name,
                song_title,
                song_filename
            FROM r4_songs
            JOIN r4_albums USING (album_id)
            WHERE song_title ILIKE %s
            AND song_verified IS TRUE
            ORDER BY album_name, song_title
        '''
        cur.execute(sql, [f'%{args.query}%'])
        rows = cur.fetchall()

    for row in rows:
        song_id = int(row[0])
        album_name = row[1]
        song_title = row[2]
        song_filename = row[3]
        print(f'{song_id} // {album_name} // {song_title} // {song_filename}')


if __name__ == '__main__':
    main()
